function tsgqc_GUI
% tsgqc_GUI
%
% TSG (Thermosalinograph) Quality Control software
%
% $Id$
%

%% COPYRIGHT & LICENSE
%  Copyright 2007 - IRD US191, all rights reserved.
%
%  This file is part of tsgqc_GUI.
%
%    Datagui is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    tsgqc_GUI is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Datagui; if not, write to the Free Software
%    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


%%  Initialization tasks
%   ********************

% Clear Command Window display, giving up a "clean screen."
% ---------------------------------------------------------
clc;

% Find program directory.
% functions (icons) should be store at a lower level
% add directories to Matlab search path, works on UNIX
% and Windows host
% ---------------------------------------------------
tsgqcname = mfilename;
fulltsgqcname = mfilename('fullpath');
DEFAULT_PATH_FILE = strrep(fulltsgqcname, tsgqcname, '') ;
p = [pathsep,...
  DEFAULT_PATH_FILE,[filesep 'tsg_util' pathsep],...
  DEFAULT_PATH_FILE,[filesep 'tsg_data' pathsep],...
  DEFAULT_PATH_FILE,[filesep 'tsg_io' pathsep]
  ];
addpath( p, '-END' );
rehash;

% get screen dimensions (pixels)
% ------------------------------
set(0,'Units','pixels');
screenSize = get(0,'ScreenSize');

% define default font size based on screen resolution
% ---------------------------------------------------
if screenSize(3) <= 1024
  tsg.fontSize = 9;
end

% Screen limits for the GUI
% -------------------------
set(0,'Units','normalized');
guiLimits = get(0,'ScreenSize');
guiLimits(1) = guiLimits(1) + 0.01;
guiLimits(2) = guiLimits(2) + 0.05;
guiLimits(3) = guiLimits(3) - 0.02;
guiLimits(4) = guiLimits(4) - 0.15;

%% Create and then hide the GUI as it is being constructed.
% --------------------------------------------------------
hMainFig = figure(...
  'Name', 'TSG Validation', ...
  'NumberTitle', 'off', ...
  'Resize', 'on', ...
  'Menubar','none', ...
  'Toolbar', 'none', ...
  'UserData', 'ButtonMotionOff', ...
  'WindowButtonMotionFcn', @MouseMotion, ...
  'CloseRequestFcn', @QuitProgram,...
  'HandleVisibility','callback',...
  'Visible','on',...
  'Units', 'normalized',...
  'Position',guiLimits, ...
  'Color', get( 0, 'DefaultUIControlBackgroundColor' ));

% Initialize tsg structure
% ------------------------
tsg_initialisation(hMainFig)

% Retrieve named application data
% -------------------------------
tsg = getappdata( hMainFig, 'tsg_data');

%%  Construct the Menu
%   -----------------
hFileMenu = uimenu(...
  'Parent', hMainFig,...
  'HandleVisibility','callback',...
  'Label', 'File');
hOpenMenu = uimenu(...
  'Parent', hFileMenu,...
  'Label','Open',...
  'Accelerator','O',...
  'HandleVisibility','callback',...
  'UserData', 'off',...
  'Callback', @OpenMenuCallback);
hSaveMenu = uimenu(...
  'Parent', hFileMenu,...
  'Label','Save',...
  'Accelerator','S',...
  'Enable', 'off',...
  'UserData', 'off',...
  'HandleVisibility','callback',...
  'Callback',@SaveMenuCallback);
hQuitMenu = uimenu(...
  'Parent',hFileMenu,...
  'Label','Quit',...
  'Separator','on',...
  'Accelerator','Q',...
  'HandleVisibility','callback',...
  'Callback',@QuitMenuCallback);

% Edit menu with Undo/Redo submenu 
% --------------------------------
hEditMenu = uimenu(hMainFig,'Label','Edit');
uimenu(hEditMenu,'Label','Undo',...
  'Accelerator','Z',...
  'Tag','Undo',...
  'Enable', 'off',...
  'Callback',@UndoMenuCallback);
uimenu(hEditMenu,'Label','Redo',...
  'Accelerator','R',...
  'Tag','Redo',...
  'Enable', 'off',...
  'Callback',@RedoMenuCallback);


%%  Construct the Toolbar
%   -----------------
hToolbar       =   uitoolbar(...   % Toolbar for Open and Print buttons
  'Parent',hMainFig, ...
  'HandleVisibility','on');
hOpenPushtool  =   uipushtool(...   % Opendoc toolbar button
  'Parent',hToolbar,...
  'TooltipString','Open file',...
  'CData', iconRead(fullfile(matlabroot, ...
  '/toolbox/matlab/icons/opendoc.mat')),...
  'HandleVisibility','callback', ...
  'Tag','PUSHTOOL_OPEN',...
  'UserData', 'off',...
  'Enable', 'on',...
  'ClickedCallback', @OpenMenuCallback);
hSavePushtool  =   uipushtool(...   % Open Save toolbar button
  'Parent',hToolbar,...
  'TooltipString','Save NetCDF file',...
  'CData',iconRead( ...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'savedoc.mat']),...
  'HandleVisibility','callback', ...
  'Tag','PUSHTOOL_SAVE',...
  'Enable', 'off',...
  'ClickedCallback', @SaveMenuCallback);
hZoomToggletool  =   uitoggletool(...   % Open Zoom toolbar button
  'Parent',hToolbar,...
  'Separator', 'on', ...
  'TooltipString','Zoom',...
  'CData', iconRead(fullfile(matlabroot, ...
  '/toolbox/matlab/icons/zoom.mat')),...
  'HandleVisibility','callback', ...
  'Tag','PUSHTOOL_ZOOM',...
  'Enable', 'off',...
  'OffCallback', @Zoom_OffMenuCallback,...
  'ONCallback',  @Zoom_OnMenuCallback);
hPanToggletool  =   uitoggletool(...   % Open Pan toolbar button
  'Parent',hToolbar,...
  'TooltipString','Pan',...
  'CData',iconRead(fullfile(matlabroot, ...
  '/toolbox/matlab/icons/pan.mat')),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_PAN',...
  'Enable', 'off',...
  'OffCallback', @Pan_OffMenuCallback,...
  'OnCallback',  @Pan_OnMenuCallback);
hQCToggletool  =   uitoggletool(...   % Open QC toolbar button
  'Parent',hToolbar,...
  'TooltipString','Validation codes',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'qcicon.mat']),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_QC',...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @QC_OffMenuCallback,...
  'OnCallback',  @QC_OnMenuCallback);
hMapToggletool  =   uitoggletool(...   % Open Map toolbar button
  'Parent',hToolbar,...
  'TooltipString','Map and ship track',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'mapicon.mat']),...
  'HandleVisibility','callback', ...
  'Tag','PUSHTOOL_MAP',...
  'UserData', 'off', ...
  'Enable', 'off',...
  'OffCallback', @Map_OffMenuCallback,...
  'OnCallback',  @Map_OnMenuCallback);
hClimToggletool  =   uitoggletool(...   % Open Climatology toolbar button
  'Parent',hToolbar,...
  'TooltipString','Climatology',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'climicon.mat']),...
  'HandleVisibility','callback', ...
  'Tag','PUSHTOOL_CLIM',...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @Clim_OffMenuCallback,...
  'OnCallback',  @Clim_OnMenuCallback);
hBottleToggletool  = uitoggletool(...   % Correction module toolbar button
  'Parent',hToolbar,...
  'TooltipString','Correct the SSS TSG data',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'bottleicon.mat']),...
  'HandleVisibility','callback', ...
  'Tag','PUSHTOOL_BOTTLE',...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @Bottle_OffMenuCallback,...
  'OnCallback',  @Bottle_OnMenuCallback);
hStartlimitToggletool  = uitoggletool(...   % Select beginning of time series toolbar button
  'Parent',hToolbar,...
  'TooltipString','Select start time',...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'startlimit.mat']),...
  'HandleVisibility','callback', ...
  'Tag', 'CORRECT_STARTTIME', ...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @SelectTime_OffMenuCallback,...
  'OnCallback',  @SelectTime_OnMenuCallback);
hEndlimitToggletool  = uitoggletool(...   % Select end of time series
  'Parent',hToolbar,...
  'TooltipString','Select end time',...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'endlimit.mat']),...
  'HandleVisibility','callback', ...
  'Tag', 'CORRECT_ENDTIME', ...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @SelectTime_OffMenuCallback,...
  'OnCallback',  @SelectTime_OnMenuCallback);

hHeaderPushtool  = uipushtool(...   % Open headerForm button
  'Parent',hToolbar,...
  'TooltipString','Fill the header form',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'hdricon.mat']),...
  'HandleVisibility','callback', ...
  'Tag','PUSHTOOL_HEADER',...
  'Enable', 'off',...
  'ClickedCallback', @HeaderMenuCallback);

%% Dynamic text area that displays the loaded filename, date, position and
%  salinity, temperature
% ------------------------------------------------------------------------

% create an uipanel
% -----------------
hInfoPanel = uipanel( ...
  'Parent', hMainFig, ...
  'Units', 'normalized', ...
  'BorderType', 'none',...
  'Visible', 'on', ...
  'Position',[.01, .96, .98, .04]);

% Dynamic text area that displays the date
% ----------------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.01, .25, .04, .5], ...
  'String', 'File:');
hInfoFileText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'No file loaded', ...
  'Position', [.05, .25, .1, .5]);

% Text area that displays the date
% --------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.16, .25, .03, .5], ...
  'String', 'Date:');
hInfoDateText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.20, .25, .13, .5]);

% Text area that displays the latitude
% ------------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.34, .25, .05, .5], ...
  'String', 'Latitude:');
hInfoLatText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.4, .25, .9, .5]);

% Text area that displays the longitude
% -------------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.50, .25, .06, .5], ...
  'String', 'Longitude:');
hInfoLongText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.57, .25, .07, .5]);

% Text area that display salinity and temperature
% -----------------------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.65, .25, .04, .5], ...
  'String', 'SSPS:');
hInfoSSPSText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.69, .25, .05, .5]);
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.75, .25, .04, .5], ...
  'String', 'SSJT:');
hInfoSSJTText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.79, .25, .05, .5]);
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.85, .25, .04, .5], ...
  'String', 'SSTP:');
hInfoSSTPText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.89, .25, .05, .5]);

%% Construct the plot axes
% -----------------------
% The axes 1,2 and 3 will be plot in a uipanel
% axe 1 as propertie 'HandleVisibility' set to 'on' to prevent hidden 
% visibility when first plot is empty
%--------------------------------------------------------------------
hPlotsPanel = uipanel( ...
  'Parent', hMainFig, ...
  'Units', 'normalized', ...
  'BorderType', 'etchedin',...
  'Visible', 'on', ...
  'Position',[0.15, 0.0, .85, .95]); 
hPlotAxes(1) = axes(...     % the axes for plotting Salinity
  'Parent', hPlotsPanel, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'HandleVisibility','on', ...  % set to callback after first plot
  'Position',[.05, .64, .93, .35]);
hPlotAxes(2) = axes(...     % the axes for plotting temperature
  'Parent', hPlotsPanel, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'HandleVisibility','callback', ...
  'Position',[.05, .32, .93, .27]);
hPlotAxes(3) = axes(...     % the axes for plotting ship velocity
  'Parent', hPlotsPanel, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'HandleVisibility','callback', ...
  'Position',[.05, .06, .93, .22]);

% The map will be plot in a uipanel
hMapPanel = uipanel( ...
  'Parent', hMainFig, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'Position',[0.15, 0, .85, .57]);
hPlotAxes(4) = axes(...     % the axes for plotting ship track map
  'Parent', hMapPanel, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'Color', 'none', ...
  'UserData', 'off', ...
  'HandleVisibility','on', ...
  'Position',[.05, .05, .9, .9]);

%% Construct Button Group panel for QC
% -------------------------------------
hbgQc = uibuttongroup(...
  'Parent', hMainFig, ...
  'Title', 'Validation Codes', ...
  'FontSize', tsg.fontSize-1, ...
  'Fontweight', 'bold', ...
  'tag', 'TAG_QC_DISPLAY_PANEL',...
  'HandleVisibility','on',...
  'Visible', 'on',...
  'BorderType', 'etchedin',...
  'Position', [.0, .71, .15, .25]);

% Initialize CallBack for button group properties
% ----------------------------------------------- 
set(hbgQc,'SelectionChangeFcn',@RadiobuttonQc);

%% Construct the context menu and RadioButtons for Quality control
% ----------------------------------------------------------------
hQcCmenu = uicontextmenu(...
  'Parent', hMainFig, ...
  'HandleVisibility','on' );

% get list of keys from hashtable tsg.qc.hash, defined inside
% tsg_initialisation.m
% -----------------------------------------------------------
qc_list = get(tsg.qc.hash);

% internal loop count with state to on
% ------------------------------------
count = 0;

% iterate (loop) on each key store inside hastable
% ------------------------------------------------
for i=1:numel(qc_list)
  
  % get key and some values in hashtable
  % ------------------------------------
  key   = qc_list{i};
  label = get(tsg.qc.hash, key, 'label');
  color = get(tsg.qc.hash, key, 'color');
  state = get(tsg.qc.hash, key, 'state');
  
  % construct context menu with only code set to 'on' (valid)
  % ---------------------------------------------------------
  if strcmp( state, 'on')
    
    % add menu to uicontextmenu
    % -------------------------
    uimenu(...
      'Parent', hQcCmenu,...
      'HandleVisibility','on', ...
      'Label', label,...
      'ForegroundColor', color,...
      'Callback', {@Qc, key});
    
    % add button QC on hbgQc Button group
    % -----------------------------------
    uicontrol(...
      'Parent', hbgQc,...
      'Style', 'radiobutton',...
      'Units', 'normalized', ...
      'Fontsize', tsg.fontSize-1, ...
      'HorizontalAlignment', 'left', ...
      'HandleVisibility','on', ...
      'String', label,...
      'Tag', ['TAG_QC_RADIO_' key], ...
      'ForegroundColor', color,...
      'Position', [.01, .85-count*.12, .6, 0.08]);
    
    % add text QC display statistic on hQcPanel
    % -----------------------------------------
    uicontrol(...
      'Parent', hbgQc,...
      'Style', 'text',...
      'Units', 'normalized', ...
      'Fontsize', tsg.fontSize-1, ...
      'HorizontalAlignment', 'right', ...
      'HandleVisibility','on', ...
      'String', 'N/A ',...
      'ForegroundColor', color,...
      'Tag', ['TAG_QC_TEXT_' key],...
      'Position', [.61, .85-count*.12, .37, 0.08]);
    
    % increment count
    % ---------------
    count = count + 1;
  end
end

%% uiPanel for the Date limits used in the correction module

% Choose the date limits for the correction
% --------------------------------------------------
% Create the uipanel
hpDateLimit = uipanel( ...
  'Parent', hMainFig, ...
  'Title', 'Date Limits', ...
  'Units', 'normalized', ...
  'FontSize', tsg.fontSize-1, ...
  'Fontweight', 'bold', ...
  'Position', [.0, .5, .15, .18]);
%  'BackgroundColor', 'white',...

htDateMin = uicontrol( ...
  'Parent', hpDateLimit, ...
  'Style', 'Text', ...
  'String', 'Min : yyyy-mm-dd hh:mm:ss', ...
  'HorizontalAlignment', 'left', ...
  'Units', 'normalized', ...
  'FontSize', tsg.fontSize-1, ...
  'Position',[.01 .8 .95 .15]);

hetDateMin = uicontrol( ...
  'Parent', hpDateLimit, ...
  'Style', 'edit', ...
  'Units', 'normalized', ...
  'BackgroundColor', 'white',...
  'FontSize', tsg.fontSize, ...
  'Position',[.01 .6 .95 .17]);

htDateMax = uicontrol( ...
  'Parent', hpDateLimit, ...
  'Style', 'Text', ...
  'String', 'Max : yyyy-mm-dd hh:mm:ss', ...
  'HorizontalAlignment', 'left', ...
  'Units', 'normalized', ...
  'FontSize', tsg.fontSize-1, ...
  'Position',[.01 .3 .95 .15]);

hetDateMax = uicontrol( ...
  'Parent', hpDateLimit, ...
  'Style', 'edit', ...
  'Units', 'normalized', ...
  'BackgroundColor', 'white',...
  'FontSize', tsg.fontSize, ...
  'Position',[.01 .1 .95 .17]);

% -------------------------
%  Callbacks for tsgcqc_GUI
%  ------------------------

%% OpenMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the Open menu item is selected
  %----------------------------------------------------------------------
  function OpenMenuCallback(hObject, eventdata)
 
    % Pointer set to watch during reading and plotting
    % ------------------------------------------------
    set( hMainFig, 'Pointer', 'watch' );

    % Open standard dialog box for retrieving files
    % ---------------------------------------------
    [fileName, pathname, filterIndex] = uigetfile( ...
      {'*.txt';'*.xml';'*.nc';'*.lbv';'*.btl'}, 'Pick a file');

    error1 = -1;
    error2 = -1;

    % flushes the event queue and updates the closed uigetfile window
    % ---------------------------------------------------------------
    drawnow;
    
    % if the user clicks the Cancel button or closes the dialog window, 
    % FileName and PathName are set to 0.
    % -----------------------------------------------------------------
    if ~isequal(fileName, 0)

      % Read the data
      % -------------
      switch filterIndex

        case 1
          error1 = readTsgDataTxt( hMainFig, fileName );
        case 2
          error1 = readTsgDataXML( hMainFig, fileName );
        case 3
          error1 = readTsgDataNetCDF( hMainFig, fileName );       
          %# a modifier
          if error1 == 1
            error2 = error1;
          end
        case 4
          error1 = readTsgDataLabview( hMainFig, fileName );  
        case 5
          error2 = readBucketData(hMainFig, fileName );
        otherwise
          return;

      end

      % A TSG file has been read. Plot the data.
      % ----------------------------------------
      if error1 ~= -1

        % A TSG file has been open and read
        % ---------------------------------
        set( hOpenMenu, 'UserData', 'on' );
        
        % enable toolbar menu pushtool
        % ----------------------------
        hdl_pushtool = findobj('-regexp','Tag', 'PUSHTOOL_');
        set(hdl_pushtool, 'Enable', 'on');
        
        % enable Save menu
        % ----------------
        set(hSaveMenu, 'Enable', 'on');

        % update the filename display
        % ---------------------------
        set( hInfoFileText, 'String', fileName);
        
        % update some fields in tsg structure
        % -----------------------------------
        updateTsgStruct(hMainFig);
        
        % Running average of TSG time series
        % ----------------------------------
        tsg_moveaverage(hMainFig);

        % The callback to detect the mouse motion can be set to on
        % --------------------------------------------------------
        set( hMainFig, 'UserData', 'ButtonMotionOn');

        % Make the Salinity, temperature and velocity plot
        % ------------------------------------------------
        plot_SalTempVel( hMainFig, hPlotAxes );

        % Plot the Map with the ship trackline
        % ------------------------------------
        plotmap( hMainFig, hPlotAxes)

      end
    end

    % Merge the different water sample (NetCdf or ASCII files) in
    % a unique structure : 'sample'
    % -----------------------------------------------------------
    %@ desactive le merge pour l'instant en attente d'une structure de
    %donn�es validee
    %@[sample] = tsg_mergesample( hMainFig );

    %@ Save the 'sample' struct. as an application data
    % ------------------------------------------------
    %if ~isempty( sample )
    %    setappdata( hMainFig, 'sample', sample );
    %end

    % Plot Salinity bucket 
    % NOT at that step. It will interfere with the QC process
    % Maybe use a new pushbutton to draw or delete the bucket
    % -------------------------------------------------------
    if error2 ~= -1
      plot_WS( hMainFig, hPlotAxes );
      plot_EXT( hMainFig, hPlotAxes );
    end
    
    % Pointer reset to arrow
    % ----------------------
    set( hMainFig, 'Pointer', 'arrow' );

  end

%% Zoom_OffMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the zoom toggle toolbar is released
  %----------------------------------------------------------------------
  function Zoom_OffMenuCallback(hObject, eventdata)
    
    % turns interactive zooming off
    % -----------------------------
    zoom off;
 
    % Set the right limit and interval to the 3 axes
    % ----------------------------------------------
    for iaxe = 1:3
      set(hPlotAxes(iaxe),'XTickMode','auto')
      datetick(hPlotAxes(iaxe),'x','keeplimits')
    end
    
    % Turns off the automatic adaptation of date ticks
    % ------------------------------------------------
    zoomAdaptiveDateTicks('off');
    
  end

%% Zoom_OnMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the zoom toggle toolbar is released
  %----------------------------------------------------------------------
  function Zoom_OnMenuCallback(hObject, eventdata)
    
    % cursor back to normal
    % ---------------------
    set(hMainFig,'Pointer','arrow');
    
    % returns a zoom mode object for the figure hMainFig handle
    % --------------------------------------------------------
    hZoom = zoom(hMainFig);
    
    % Disallows a zoom operation on the MAP axes objects
    % --------------------------------------------------
    setAllowAxesZoom(hZoom, hPlotAxes(4), false);

    % turns on interactive zooming (same effect than zoom on) but prevent
    % side effect on another figure
    % -------------------------------------------------------------------
    set(hZoom,'enable', 'on');
    %zoom on;
    
    % Turns on the automatic adaptation of date ticks
    % -----------------------------------------------
    zoomAdaptiveDateTicks('on');
 
    % Desactivate some toggle buttons
    % -------------------------------
    set( hQCToggletool,  'state', 'off' );
    set( hPanToggletool, 'state', 'off' );
    set( hStartlimitToggletool, 'State', 'off' );
    set( hEndlimitToggletool,   'State', 'off' );
   
  end

%% Pan_OffMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the pan toggle toolbar is released
  %----------------------------------------------------------------------
  function Pan_OffMenuCallback(hObject, eventdata)
    
    % turns interactive pan off
    % -------------------------
    pan off;
    
    % Set the right limit and interval to the 3 axes
    % ----------------------------------------------
    for iaxe = 1:3
      set(hPlotAxes(iaxe),'XTickMode','auto')
      datetick(hPlotAxes(iaxe),'x','keeplimits')
    end
 
    % Turns off the automatic adaptation of date ticks
    % ------------------------------------------------
    panAdaptiveDateTicks('off');
    
  end

%% Pan_OnMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the pan toggle toolbar is selected
  %----------------------------------------------------------------------
  function Pan_OnMenuCallback(hObject, eventdata)
    
    % cursor back to normal
    % ---------------------
    set(hMainFig,'Pointer','arrow');
    
    % Returns a pan mode object for the figure handle
    % -----------------------------------------------
    hPan = pan(hMainFig);
    
    % Disallows a pan operation on the MAP axes objects
    % --------------------------------------------------
    setAllowAxesPan(hPan, hPlotAxes(4), false);

    % turns on interactive pan (same effect than pan on) but prevent
    % side effect on another figure
    % --------------------------------------------------------------
    set(hPan,'enable', 'on');
    
    % Turns on the automatic adaptation of date ticks
    % -----------------------------------------------
    panAdaptiveDateTicks('on');
    
    % Desactivate some toggle buttons
    % -------------------------------
    set( hQCToggletool,   'state', 'off' );
    set( hZoomToggletool, 'state', 'off' );
    set( hStartlimitToggletool, 'State', 'off' );
    set( hEndlimitToggletool,   'State', 'off' );
    
  end

%% QC_OffMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the QC toggle tool is released
  %----------------------------------------------------------------------
  function QC_OffMenuCallback(gcbo, eventdata)
    
    % Desactive right clic menu on first axes (salinity)
    % ---------------------------------------------------
    set(hPlotAxes(1),'UIContextMenu', []);
    
    % Desactive clic mouse menu (zoom) on first axes (salinity)
    % ----------------------------------------------------------
    set(hPlotAxes(1),'ButtonDownFcn', []);
    
    % cursor back to normal
    % ---------------------
    set(hMainFig,'Pointer','arrow');
    
  end

%% QC_OnMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the QC toggle tool is pressed
  %----------------------------------------------------------------------
  function QC_OnMenuCallback(gcbo, eventdata)
    
    % Desactivate Zoom and Pan functions.
    % ----------------------------------
    set(hZoomToggletool, 'state', 'off' );
    set(hPanToggletool, 'state', 'off' );
    
    % Activate right clic context menu on first axes (salinity)
    % ---------------------------------------------------------
    set(hPlotAxes(1),'UIContextMenu', hQcCmenu);

    % Activate clic mouse menu on first axes (salinity) for next rbbox
    % ----------------------------------------------------------------
    set(hPlotAxes(1),'ButtonDownFcn', @QC_SelectCallback);
    
    % change cursor to crosshair aspect
    % ---------------------------------
    set( hMainFig, 'Pointer', 'crosshair');
    
    % Active keyPress/release callback in QC mode for PAN and Zoom
    % ------------------------------------------------------------
    %set( hMainFig, 'KeyReleaseFcn',  @QC_keyReleaseCallback);
    %set( hMainFig, 'KeyReleaseFcn',@QC_keyReleaseCallback);
    
    % -------------------------------------------------------------
    % nested function on mouse clic when QC toggle tool is selected
    % -------------------------------------------------------------
    function QC_SelectCallback(gcbo, eventdata)

      % disable ButtonMotion on main fig during select
      % prevent drawing to map
      % ----------------------------------------------
      set( hMainFig, 'WindowButtonMotionFcn', []);
      
      % Retrieve named application data
      % -------------------------------
      tsg = getappdata( hMainFig, 'tsg_data');
      
      % Selection of the data within the figure
      % ---------------------------------------
      point1 = get(gca,'CurrentPoint');    % button down detected
      finalRect = rbbox;                   % return figure units
      point2 = get(gca,'CurrentPoint');    % button up detected

      point1 = point1(1,1:2);              % extract x and y
      point2 = point2(1,1:2);
      
      p1 = min(point1,point2);
      p2 = max(point1,point2);             % calculate locations

      % get index on selected zone
      % ---------------------------
      ind = find(tsg.DAYD > p1(1,1) & tsg.DAYD < p2(1,1) & ...
                 tsg.SSPS > p1(1,2) & tsg.SSPS < p2(1,2));

      tsg.SSPS_QC(ind) = tsg.qc.active.Code;
      
      % put last SSPS_QC in queue (Undo/Redo)
      % -------------------------------------
      tsg.queue = push( tsg.queue, tsg.SSPS_QC);

      % set current axes and plot
      % -------------------------
      axes(hPlotAxes(1));
      hold on
      color = ['.' tsg.qc.active.Color];
      
      % plot selected data with selected code
      % --------------------------------------
      plot(tsg.DAYD(ind), tsg.SSPS(ind), color );
      hold off

      % As soon as a modification took place the data should be saved
      % -------------------------------------------------------------
      set( hSaveMenu, 'UserData', 'on' );
      
      % Save the modifications
      % ----------------------
      setappdata( hMainFig, 'tsg_data', tsg);
      
      % refresh QC statistic panel
      % --------------------------
      display_QC( hMainFig, hPlotAxes );
      
      % enable ButtonMotion on main fig after select QC area
      % ----------------------------------------------------
      set( hMainFig, 'WindowButtonMotionFcn', @MouseMotion);
    end
    
    % -------------------------------------------------------------
    % nested function on keypress when QC selected
    % -------------------------------------------------------------
%     function QC_keyReleaseCallback(gcbo, evnt)
%       out = sprintf('Character: %c\nKey: %s\n',evnt.Character,evnt.Key);
%             disp(out)
%        if strcmp(evnt.Key, 'control')
%          panState = get(hZoomToggletool, 'state');
%          if strcmp(panState, 'on')
%            set(hZoomToggletool, 'state', 'off' );
%            set(hQCToggletool,   'state', 'on' );
%          else
%            set(hZoomToggletool, 'state', 'on' );
%          end
%        end
%        set( hMainFig, 'KeyReleaseFcn',  @QC_keyReleaseCallback);
%     end
    
    % -------------------------------------------------------------
    % nested function on keyRelease when QC selected
    % -------------------------------------------------------------
%     function QC_keyReleaseCallback(gcbo, event)
%        if strcmp(event.Key, 'control')
%          set(hPanToggletool, 'state', 'off' );
%        end
%     end
    
  end

%% Qc context menu selected
  %----------------------------------------------------------------------
  % Callback function run when the QC context menu is selected
  %---------------------------------------------------------------------
  function Qc(hObject, eventdata, key)  
    
      % Retrieve Default Quality Code and Color
      % ---------------------------------------
      tsg = getappdata( hMainFig, 'tsg_data');
      
      % get key and some values in hashtable
      % ------------------------------------
      code  = get(tsg.qc.hash, key, 'code');
      color = get(tsg.qc.hash, key, 'color');
       
      % set active code and color from selected context menu
      % ----------------------------------------------------
      tsg.qc.active.Code  = code;
      tsg.qc.active.Color = color;

      setappdata( hMainFig, 'tsg_data', tsg );
            
  end

%% Radiobutton Quality Control
  % ---------------------------------------------------------------
  % Callback to select CODE and COLOR QC from the RadioButton Group
  % ---------------------------------------------------------------
  function RadiobuttonQc(source, eventdata)
        
      % Retrieve Default Quality Code and Color
      % ---------------------------------------
      tsg = getappdata( hMainFig, 'tsg_data');
      
      % Retrieve the Kye frome the active RadioButton
      % ---------------------------------------------
      rbTag = get(eventdata.NewValue,'Tag');
      key = strrep(rbTag, 'TAG_QC_RADIO_', '');
      
      % get key and some values in hashtable
      % ------------------------------------
      code  = get(tsg.qc.hash, key, 'code');
      color = get(tsg.qc.hash, key, 'color');
       
      % set active code and color from selected context menu
      % ----------------------------------------------------
      tsg.qc.active.Code  = code;
      tsg.qc.active.Color = color;

      setappdata( hMainFig, 'tsg_data', tsg );

  end

%% MouseMotion
  %---------------------------------------------------------------------
  % Callback function run when mouse pointer is moving on temperature plot
  % draw corresponding measurement position on map
  %---------------------------------------------------------------------
  function MouseMotion(hObject, eventdata)

    % Test if the callback can be activated
    % -------------------------------------
    if strcmp( get( hMainFig, 'UserData'), 'ButtonMotionOn')

      % Retrieve named application data
      % -------------------------------
      tsg = getappdata( hMainFig, 'tsg_data');

      % Get current position of cusor and return its coordinates in
      % axes with handle h_axes
      % -----------------------------------------------------------
      [x, y] = gpos(hPlotAxes(1));
      
      % Code to Activate the PAN function when QC mode is active
      % A PAN zone is defined in the bottom (5%) of PlotAxes(1)
      % 2 callback are needed : 
      %    1 - one to desactivate QC when Pan is set to on.
      %    2 - one to reactivate QC once the pan has been used.
      % ---------------------------------------------------------
      qcState = get(hQCToggletool,   'state' );
      if strcmp(qcState, 'on' );
        
        limx = get(hPlotAxes(1), 'XLim');
        limy = get(hPlotAxes(1), 'YLim');
        
        % Suppose that Y axes is increasing from the bottom to the top
        % ------------------------------------------------------------
        limy2 = limy(1) + (limy(2)-limy(1)) * 0.05;

        if  x > limx(1) && x < limx(2) &&  y <= limy2
          hPan = pan(hMainFig);
          set(hPan,'ActionPreCallback',  @prePanCallback);
          set(hPan,'ActionPostCallback', @postPanCallback);
          set(hPan,'Enable','on');
        else
          pan off
        end
      end
      
      % Dynamically display data in uicontrol
      % -------------------------------------
      if x > tsg.DAYD(1) && x < tsg.DAYD(end)
        indCursor = find( tsg.DAYD > x);
        set( hInfoDateText, 'String',...
          datestr(tsg.DAYD(indCursor(1)),'dd/mm/yyyy   HH:MM'));
        set( hInfoLatText,  'String', dd2dm(tsg.LATX(indCursor(1)),0));
        set( hInfoLongText, 'String', dd2dm(tsg.LONX(indCursor(1)),1));
        set( hInfoSSPSText, 'String', tsg.SSPS(indCursor(1)));
        set( hInfoSSJTText, 'String', tsg.SSJT(indCursor(1)));
        if ~isempty(tsg.SSTP)
          set( hInfoSSTPText, 'String', tsg.SSTP(indCursor(1)));
        end

        % Plot the position on the map if this one is active
        % --------------------------------------------------
        if strcmp( get(hMapPanel, 'Visible'), 'on')

          % Select the map axes
          % -------------------
          axes( hPlotAxes(4));

          if isempty( get(hMapPanel, 'UserData'))
            hMarker = m_line( ...
              tsg.LONX(indCursor(1)), tsg.LATX(indCursor(1)),...
              'Marker','o','MarkerSize',5, ...
              'Color','r', 'MarkerFaceColor','r');
            set(hMapPanel, 'UserData', hMarker)
          else
            delete(get(hMapPanel, 'UserData'));
            hMarker = m_line( ...
              tsg.LONX(indCursor(1)), tsg.LATX(indCursor(1)),...
              'Marker','o','MarkerSize',5, ...
              'Color','r', 'MarkerFaceColor','r');
            set(hMapPanel, 'UserData', hMarker);
          end
          
        end
      end
    end
  end
%% prePanCallback
function prePanCallback(obj, evd)
  
  set(hQCToggletool, 'state', 'off' );

end

%% postPanCallback
function postPanCallback(obj, evd)
  
  set(hQCToggletool, 'state', 'on' );
  
  % Set the right limit and interval to the 3 axes
  % ----------------------------------------------
  for iaxe = 1:3
    set(hPlotAxes(iaxe),'XTickMode','auto')
    datetick(hPlotAxes(iaxe),'x','keeplimits')
  end
  
end

%% Map_OffMenuCallback
  %---------------------------------------------------------------------
  % Callback function run when the Map tool bar item is unselected
  %---------------------------------------------------------------------
  function Map_OffMenuCallback(hObject, eventdata)
    
    % Make the earth map invisible
    % ----------------------------
    set(hMapPanel, 'Visible', 'off' );   
    
  end

%% Map_OnMenuCallback
  %---------------------------------------------------------------------
  % Callback function run when the Map tool bar item is selected
  %---------------------------------------------------------------------
  function Map_OnMenuCallback(hObject, eventdata)
     
    % Desactivate Zoom and Pan toggle buttons
    % may be change in the futur ...
    % ---------------------------------------
    set(hZoomToggletool,  'state', 'off' );
    set(hPanToggletool,  'state', 'off' );
    
    % Make the earth map visible
    % --------------------------
    set(hMapPanel, 'Visible', 'on' );
    
  end

%% Bottle_OnMenuCallback : Correction Module
  %---------------------------------------------------------------------
  % Callback function run when the bootle push tool is selected
  %---------------------------------------------------------------------
  function Bottle_OnMenuCallback(hObject, eventdata)
    % Callback function run when the Bottle tool bar item is selected

    % Get the tsg structure
    % ---------------------
    tsg = getappdata(hMainFig, 'tsg_data');
    
    % Desactivate the Climatology button
    % ----------------------------------
    set( hClimToggletool, 'Enable', 'off');
    
    % Activate some toolbar buttons
    % -----------------------------
    hdl_Toggletool = findobj('-regexp','Tag', 'CORRECT_');
    set(hdl_Toggletool, 'Enable', 'on');
    
    % Test if tsg and sample data have been loaded
    % --------------------------------------------
    if ~isempty( tsg.SSPS_WS ) || ~isempty( tsg.SSPS_EXT )
      
      % Merge bucket and external samples
      % ---------------------------------
      tsg_mergesample( hMainFig );
                    
      % Compute the sample-TSG differences
      % ----------------------------------
      diffTsgSample( hMainFig );

      % plot Salinity Difference
      % ------------------------
      plot_Sample( hMainFig, hPlotAxes );
      
      % Get the information on time limits of the time series
      % Write them in the uipanel
      % -----------------------------------------------------
      set( hetDateMin, 'String', datestr(tsg.DAYD(1), 31));
      set( hetDateMax, 'String', datestr(tsg.DAYD(end), 31)); 

    else
      msgbox('Sample data not loaded in the program', 'modal');
    end
  end

%% Bottle_OffMenuCallback : Correction module
  %---------------------------------------------------------------------
  % Callback function run when the bootle push tool is selected
  %---------------------------------------------------------------------
  function Bottle_OffMenuCallback(hObject, eventdata)
 
    % Activate the Climatology button
    % -------------------------------
    set( hClimToggletool, 'Enable', 'on');
    
    % Desactivate some toolbar buttons
    % --------------------------------
    hdl_Toggletool = findobj('-regexp','Tag', 'CORRECT_');
    set(hdl_Toggletool, 'state', 'off');

    % Plot the SSS, SST and Velocity
    % ------------------------------
    plot_SalTempVel( hMainFig, hPlotAxes );
    plot_WS( hMainFig, hPlotAxes );
    plot_EXT( hMainFig, hPlotAxes );
 
  end

%% SelectTime_OnMenuCallback : Correction module
%----------------------------------------------------------------------
  function SelectTime_OnMenuCallback(hObject, eventdata)
    % Callback function run when the ....
    
    % Create a pointer to select the time limits
    % ------------------------------------------
    selTimeP = ones(16)+1;
    selTimeP(1,:) = 1; selTimeP(16,:) = 1;
    selTimeP(:,1) = 1; selTimeP(:,16) = 1;
    selTimeP(1:4,8:9) = 1; selTimeP(13:16,8:9) = 1;
    selTimeP(8:9,1:4) = 1; selTimeP(8:9,13:16) = 1;
    selTimeP(5:12,5:12) = NaN; 				% Create a transparent region in the center
        
    % Desactivate Zoom and Pan functions.
    % ----------------------------------
    set(hZoomToggletool, 'state', 'off' );
    set(hPanToggletool,  'state', 'off' );

    % Toggle the state of the Qc Togglebutton to 'on' or 'off'
    % ----------------------------------------------------
    if hObject == hEndlimitToggletool
      set( hStartlimitToggletool, 'State', 'off' );
    else
      set( hEndlimitToggletool,   'State', 'off' );
    end
    
    % Activate clic mouse menu on first axes (salinity) for next rbbox
    % DO NOT USE :
    % ButtonDownFcn -  the callback executes whenever you press a button
    % while the pointer is within the axes, but not over another graphics
    % object parented to the axes.
    % -------------------------------------------------------------------
    %set(hPlotAxes(1),'ButtonDownFcn', @Time_SelectCallback);
    set( hMainFig,'WindowButtonDownFcn', @Time_SelectCallback);  
 
    % change cursor to crosshair aspect
    % ---------------------------------
    set( hMainFig, 'Pointer', 'custom', 'PointerShapeCData', selTimeP, ...
                                        'PointerShapeHotSpot',[9 9]);

    % -------------------------------------------------------------
    % nested function on mouse clic when Select Time toggle tool is selected
    % -------------------------------------------------------------
    function Time_SelectCallback(gcbo, eventdata)

      % disable ButtonMotion on main fig during select
      % prevent drawing to map
      % ----------------------------------------------
      %set( hMainFig, 'WindowButtonMotionFcn', []);

      % Get the current mouse position
      % ------------------------------
      [x, y] = gpos(hPlotAxes(1));
      a = get(hPlotAxes(1), 'CurrentPoint');
      xx = a(2,1);

      % Write the date in the Editable uicontrol
      % ----------------------------------------
      if hObject == hEndlimitToggletool
        set( hetDateMax, 'String', datestr(x, 31));
      else
        set( hetDateMax, 'String', datestr(xx, 31));
        set( hetDateMin, 'String', datestr(x, 31));
      end

      % Enable ButtonMotion on main fig 
      % -------------------------------
      %set( hMainFig, 'WindowButtonMotionFcn', @MouseMotion);
      
    end
  end

%% SelectTime_OffMenuCallback : Correction module
%----------------------------------------------------------------------
  function SelectTime_OffMenuCallback(hObject, eventdata)
  % Callback function run when the ....
  
    if hObject == hEndlimitToggletool
      set( hEndlimitToggletool, 'State', 'off');
    else
      set( hStartlimitToggletool, 'State', 'off');
    end

    set(hMainFig,'Pointer','arrow');

  end

%% Clim_OffMenuCallback
  %------------------------------------------------------------------------
  % Callback function run when the Levitus climatology toolbar is unselected
  %------------------------------------------------------------------------
  function Clim_OffMenuCallback(hObject, eventdata)

    % Get line handles
    % ----------------------------------------------
    hLine1 = get( hPlotAxes(1), 'UserData');
    hLine2 = get( hPlotAxes(2), 'UserData');
    
    % Delete climatology lines on axes
    % -----------------------------------
    try
      delete(hLine1.stdClimMinus);
      delete(hLine1.stdClimPlus);
      delete(hLine1.meanClim);
      delete(hLine2.stdClimMinus);
      delete(hLine2.stdClimPlus);
      delete(hLine2.meanClim);
      
    % if handle is missing, prevent warning message on console
    % --------------------------------------------------------
    catch
    end
    
  end

%% Clim_OnMenuCallback
  %------------------------------------------------------------------------
  % Callback function run when the Levitus climatology toolbar is unselected
  %------------------------------------------------------------------------
  function Clim_OnMenuCallback(hObject, eventdata)

    % Test if the TSG and bucket files have been read
    % -----------------------------------------------
    if strcmp( get(hOpenMenu, 'UserData'), 'on' )

      % Get data
      % -----------------------
      tsg = getappdata( hMainFig, 'tsg_data' );

      % Get line handles
      % ----------------
      hLine1 = get( hPlotAxes(1), 'UserData');
      hLine2 = get( hPlotAxes(2), 'UserData');

      % Read Climatology
      % ----------------
      if isempty( tsg.levitus.data)
        levitus = read_file_woa01( 'woa01an.nc' );
        if ~isstruct(levitus)
          errordlg({'NetCDF climatology file not present in you path',...
            'Check your matlab path or download it from',...
            'ftp://ftp.ifremer.fr/ifremer/ird/us191/oceano/lib/matlab/woa01an.nc'},...
            'NetCDF climatology file access error');
          return;
        else
          tsg.levitus.data = levitus;
          tsg.levitus.type = 'woa01an';
        end
        setappdata( hMainFig, 'tsg_data', tsg );
      end

      % round positive latitude and Longitude toward zero
      % -------------------------------------------------
      ind = find(tsg.LATX > 0);
      lat(ind) = fix(tsg.LATX(ind)) + 0.5;

      ind = find(tsg.LONX > 0);
      lon(ind) = fix(tsg.LONX(ind)) + 0.5;

      % rounds negative latitude and Longitudeto the nearest lowest integers
      % ---------------------------------------------------------------------
      ind = find(tsg.LATX <= 0);
      lat(ind) = floor(tsg.LATX(ind)) + 0.5;

      ind = find(tsg.LONX <= 0);
      lon(ind) = floor(tsg.LONX(ind)) + 0.5;

      % Calculates differences between adjacent elements of X.
      % 0 if adajacent latitude or longitude are equal
      % - 1 or -1 otherwise
      % ------------------------------------------------------------
      lat_diff = [diff( lat )'; 0];
      lon_diff = [diff( lon )'; 0];

      % Select latitude and longitude
      % -----------------------------
      ind  = find(abs(lat_diff) == 1 | abs(lon_diff == 1));
      lat2 = lat( ind );
      lon2 = lon( ind );
      time = tsg.DAYD( ind );
      temp = tsg.SSJT( ind );
      psal = tsg.SSPS( ind );

      % Get Climatology
      %           LATX(80)  = -0.5 et LATX(81)  = 0.5
      %           LONX(180) = -0.5 et LONX(181) = 0.5
      % ----------------
      axes( hPlotAxes(1) );
      mean_temp = zeros(size(ind));
      mean_psal = zeros(size(ind));
      std_temp  = zeros(size(ind));
      std_psal  = zeros(size(ind));
      for ii=1:length(ind)
        ilat         = find(tsg.levitus.data.WOA01_LATX == lat2(ii));
        ilon         = find(tsg.levitus.data.WOA01_LONX == lon2(ii));
        mean_temp(ii) = tsg.levitus.data.WOA01_MEAN_TEMP(ilat,ilon,1);
        mean_psal(ii) = tsg.levitus.data.WOA01_MEAN_PSAL(ilat,ilon,1);
        std_temp(ii)  = tsg.levitus.data.WOA01_STD_TEMP(ilat,ilon,1);
        std_psal(ii)  = tsg.levitus.data.WOA01_STD_PSAL(ilat,ilon,1);
      end

      % Plot mean salinity climatology
      % ------------------------------
      hLine1.meanClim = line( ...
        time, mean_psal,'Linestyle', '-', 'Color','k');
      
      % Plot with 3 standard deviation
      % ------------------------------
      hLine1.stdClimPlus = line( ...
        time,  mean_psal + 3 * std_psal ,'Linestyle', '-', 'Color','r');
      hLine1.stdClimMinus = line( ...
        time,  mean_psal - 3 * std_psal ,'Linestyle', '-', 'Color','r');
      
      % Plot mean temperature climatology
      % ---------------------------------
      axes( hPlotAxes(2));
      hLine2.meanClim = line( ...
        time, mean_temp,'Linestyle', '-', 'Color','k');
      hLine2.stdClimPlus = line( ...
        time,  mean_temp + 3 * std_temp ,'Linestyle', '-', 'Color','r');
      hLine2.stdClimMinus = line( ...
        time,  mean_temp - 3 * std_temp ,'Linestyle', '-', 'Color','r');

      % Store the handle of the bucketline
      % ----------------------------------
      set( hPlotAxes(1), 'UserData', hLine1 );
      set( hPlotAxes(2), 'UserData', hLine2 );
    end
  end

%% HeaderMenuCallback
  % -------------------------------------------------------------------
  % Callback function run when the headerForm tool bar item is selected
  % -------------------------------------------------------------------
  function HeaderMenuCallback(hObject, eventdata)
   
    % call header form function
    % -------------------------
    headerForm(hMainFig);
    
  end

%% SaveMenuCallback
  % -------------------------------------------------------------------
  % Callback function run when the Save menu item is selected
  % -------------------------------------------------------------------
  function SaveMenuCallback(hObject, eventdata)
    
    % Retrieve named application data
    % -------------------------------
    tsg = getappdata( hMainFig, 'tsg_data');

    % get fileName without extension in cell
    % --------------------------------------
    file = textscan( tsg.file.name,'%s','delimiter','.');

    % fill or append header form
    % -------------------------
    error = headerForm(hMainFig);

    % if user press continue button, ask for netcdf file
    % ------------------------------------------------
    if error ~= -1
      [fileName, pathName, filterIndex] = uiputfile('*.nc', ...
        'Save file name', [file{1}{1} '.nc']);

      % if user press cancel button, all var set to zero
      % ------------------------------------------------
      if fileName == 0
        return;
      end

      % Pointer set to watch during reading and plotting
      % ------------------------------------------------
      set( hMainFig, 'Pointer', 'watch' );

      % flushes the event queue and updates the closed uiputfile window
      % ---------------------------------------------------------------
      drawnow;

      % write netcdf file
      % -----------------
      error = writeTSGDataNetCDF( hMainFig, [pathName fileName] );

      % Pointer reset to arrow
      % ----------------------
      set( hMainFig, 'Pointer', 'arrow' );

      % Check for a writing pb to NetCDF
      % must to be rewriting
      % --------------------------------
      if error == -1
        warning('NetCDF writing error: %s %s', pathName, fileName);
        return;
      end
      
      % update the display
      % ------------------
      set( hInfoFileText, 'String', fileName);
      
      % enable Quality Control mode
      % ---------------------------
      hdl_pushtool = findobj('Tag', 'QC');
      set(hdl_pushtool, 'Enable', 'on');
    end
    
  end

%% QuitMenuCallback
  % -----------------------------------------------------------------
  % Callback function run when the Quit menu item is selected
  % -----------------------------------------------------------------
  function QuitMenuCallback(hObject, eventdata)  

    % If the data have been modified and not save, the program
    % ask to save the data
    % --------------------------------------------------------
    if  strcmp( get( hSaveMenu, 'UserData' ), 'on')
      selection = ...
        questdlg('The file has been modified.  Do you want to save it ?',...
        'Save before Quit?',...
        'Yes', 'No', 'Yes');
      if strcmp(selection, 'Yes')
        return;
      else
        QuitProgram;
      end
    else
      selection = ...
        questdlg(['Quit ' get(hMainFig, 'Name') '?'],...
        ['Quit ' get(hMainFig, 'Name') '?'],...
        'Yes', 'No', 'Yes');
      if strcmp(selection, 'No')
        return;
      else
        QuitProgram;
      end
    end

  end

%% UndoMenuCallback
  % -----------------------------------------------------------------------
  % Callback function run when the Edit/Undo menu item is selected (Ctrl+Z)
  % -----------------------------------------------------------------------
  function UndoMenuCallback(hObject, eventdata)  
 
    % Undo module not yet implemented
    % -------------------------------
    msgbox('Undo module not yet implemented', 'modal');
    
  end

%% RedoMenuCallback
  % -----------------------------------------------------------------------
  % Callback function run when the Edit/Redo menu item is selected (Ctrl+R)
  % -----------------------------------------------------------------------
  function RedoMenuCallback(hObject, eventdata)  
 
    % Redo module not yet implemented
    % -------------------------------
    msgbox('Redo module not yet implemented', 'modal');
    
  end

%% QuitProgram
  % -----------------------------------------------------------------
  % call from: 
  %   QuitMenuCallback
  %   callback 'CloseRequestFcn', @QuitProgram
  % -----------------------------------------------------------------
  function QuitProgram(hObject, eventdata)
    
    % close the main windows
    % ----------------------
    delete(hMainFig);

    % reset Matlab search path to default
    % -----------------------------------
    rmpath( [DEFAULT_PATH_FILE filesep 'tsg_util'] );
    rmpath( [DEFAULT_PATH_FILE filesep 'tsg_data'] );
    rmpath( [DEFAULT_PATH_FILE filesep 'tsg_io'] );
    
    % Refresh file system caches
    % --------------------------
    rehash;
    
  end

% end of tsgqc_GUI
% ----------------
end  
